@@ -1,22 +1,18 @@
-import re
-import unidecode
-import requests
-import os
-import json
-import pkg_resources
 import copy
-
-import urllib.parse
-from urllib.parse import urlparse
+import json
+import os
+import re
+from urllib.parse import urlencode, urlparse
 
 import flask
-from flask import make_response, request, render_template
 import jinja2.exceptions
+import pkg_resources
+import requests
+import unidecode
+from flask import make_response, render_template, request
 
-from . import config
-from . import navigation
-from . import card_list
-
+from . import card_list, config
+from .navigation import HELPER
 
 version = pkg_resources.Environment()["barrins-codex"][0].version
 if version[-5:] == ".dev0":  # To pass tests on commits during development
@@ -37,11 +33,47 @@ else:
 
 BASE_CONTEXT = {
     "version": version,
+    "pilotes_habitue": [
+        page for page in HELPER if HELPER.get(page, {}).get("cat") == "habitue"
+    ],
+    "pilotes_invite": [
+        page for page in HELPER if HELPER.get(page, {}).get("cat") == "invite"
+    ],
+    "decks_agro": [
+        page for page in HELPER if HELPER.get(page, {}).get("cat") == "agro"
+    ],
+    "decks_tempo": [
+        page for page in HELPER if HELPER.get(page, {}).get("cat") == "tempo"
+    ],
+    "decks_controle": [
+        page for page in HELPER if HELPER.get(page, {}).get("cat") == "controle"
+    ],
+    "decks_combo": [
+        page for page in HELPER if HELPER.get(page, {}).get("cat") == "combo"
+    ],
+    "decks_midrange": [
+        page for page in HELPER if HELPER.get(page, {}).get("cat") == "midrange"
+    ],
+    "decks_ban": [page for page in HELPER if HELPER.get(page, {}).get("cat") == "ban"],
+    "needs_crop": [page for page in HELPER if HELPER.get(page, {}).get("crop")],
+    "matchs_0110": [
+        page for page in HELPER if HELPER.get(page, {}).get("cat") == "01-10"
+    ],
+    "matchs_1120": [
+        page for page in HELPER if HELPER.get(page, {}).get("cat") == "11-20"
+    ],
+    "matchs_2130": [
+        page for page in HELPER if HELPER.get(page, {}).get("cat") == "21-30"
+    ],
+    "articles": [
+        page for page in HELPER if HELPER.get(page, {}).get("cat") == "article"
+    ],
 }
 
 
 def main():
-    # print(navigation.HELPER)
+    # print(HELPER)
+    # print(BASE_CONTEXT)
     app.run()
 
 
@@ -49,7 +81,8 @@ def main():
 @app.errorhandler(jinja2.exceptions.TemplateNotFound)
 @app.errorhandler(404)
 def page_not_found(error):
-    return flask.render_template("404.html"), 404
+    context = copy.copy(BASE_CONTEXT)
+    return flask.render_template("404.html", **context), 404
 
 
 # Static file redirection
@@ -71,7 +104,7 @@ def sitemap():
     host_components = urlparse(request.host_url)
     host_base = host_components.scheme + "://" + host_components.netloc
 
-    urls = [host_base + p["self"].url for p in navigation.HELPER.values()]
+    urls = [host_base + p["self"].url for p in HELPER.values()]
 
     xml_sitemap = render_template("sitemap.xml", urls=urls, host_base=host_base)
     response = make_response(xml_sitemap)
@@ -105,7 +138,7 @@ def index(page=None):
 def _build_url(page, _anchor=None, **params):
     url = page.url
     if params:
-        url += "?" + urllib.parse.urlencode(params)
+        url += "?" + urlencode(params)
     if _anchor:
         url += "#" + _anchor
     return url
@@ -135,33 +168,27 @@ def linker():
         path = path[:-1]
 
     def _url(page, _anchor=None, **params):
-        return _build_url(
-            navigation.HELPER.get(page, {}).get("self"), _anchor=_anchor, **params
-        )
+        return _build_url(HELPER.get(page, {}).get("self"), _anchor=_anchor, **params)
 
     def link(page, name=None, _anchor=None, **params):
         return _link(
-            navigation.HELPER.get(page, {}).get("self"),
+            HELPER.get(page, {}).get("self"),
             name=name,
             _anchor=_anchor,
             **params,
         )
 
     def name(page, name=None):
-        return name or navigation.HELPER.get(page, {}).get("self").name
+        return name or HELPER.get(page, {}).get("self").name
 
     def top():
-        return _link(navigation.HELPER.get(path, {}).get("top"), _class="text-reset")
+        return _link(HELPER.get(path, {}).get("top"), _class="text-reset")
 
     def next():
-        return _link(
-            navigation.HELPER.get(path, {}).get("next"), _class="next text-reset"
-        )
+        return _link(HELPER.get(path, {}).get("next"), _class="next text-reset")
 
     def prev():
-        return _link(
-            navigation.HELPER.get(path, {}).get("prev"), _class="prev text-reset"
-        )
+        return _link(HELPER.get(path, {}).get("prev"), _class="prev text-reset")
 
     def external(url, name, _class=None):
         if _class:
@@ -174,8 +201,8 @@ def linker():
 
     def title():
         try:
-            if navigation.HELPER.get(path, {}).get("self").path != "":
-                name = navigation.HELPER.get(path, {}).get("self").name
+            if HELPER.get(path, {}).get("self").path != "":
+                name = HELPER.get(path, {}).get("self").name
                 return f"Barrin's Codex - {name}"
         except AttributeError:
             pass
@@ -183,8 +210,8 @@ def linker():
 
     def page_name():
         try:
-            if navigation.HELPER.get(path, {}).get("self").path != "":
-                name = navigation.HELPER.get(path, {}).get("self").name
+            if HELPER.get(path, {}).get("self").path != "":
+                name = HELPER.get(path, {}).get("self").name
                 return f"{name}"
         except AttributeError:
             pass
@@ -205,15 +232,6 @@ def linker():
 
 @app.context_processor
 def display_card():
-    def get_card(name, firstprint=True):
-        fuzzy = re.sub(r"[^\w\s]", "", name)
-        fuzzy = re.sub(r"\s+", "-", fuzzy)
-        url = f"https://api.scryfall.com/cards/search?q={fuzzy}"
-        if firstprint:
-            url = url + "+is%3Afirstprint"
-        r = requests.get(url)
-        return r.json()
-
     def _name(name):
         name = name
         name = unidecode.unidecode(name).lower()
@@ -221,47 +239,61 @@ def display_card():
         return name
 
     def card_name_from_page(name):
-        page_name = navigation.HELPER.get(name, {}).get("self").name
+        page_name = HELPER.get(name, {}).get("self").name
         if "❌" in page_name:
             page_name = page_name[2:]
         return page_name
 
     def img_crop(name, front=True):
         card = CARDS[_name(name)]
+        query = "format=image&version=art_crop"
         if "faces" in card:
             if not front:
-                card = get_card(name)["data"][0]
-                return card["card_faces"][1]["image_uris"]["art_crops"]
-            return (
-                "https://api.scryfall.com/cards/"
-                + f"{CARDS[card['faces'][0]]['id']}"
-                + "?format=image&version=art_crop"
-            )
-        return (
-            "https://api.scryfall.com/cards/"
-            + f"{card['id']}?format=image&version=art_crop"
+                query = query + "&face=back"
+        return "https://api.scryfall.com/cards/" + f"{card['id']}?{query}"
+
+    def img_card(name, front=True):
+        card = CARDS[_name(name)]
+        query = "format=image&version=png"
+        if "faces" in card:
+            if not front:
+                query = query + "&face=back"
+        return f"https://api.scryfall.com/cards/{card['id']}?{query}"
+
+    def card_image(name, front=True):
+        return flask.Markup(
+            '<img src="'
+            + img_card(name, front)
+            + '" alt="'
+            + name
+            + '" class="col-md-3 float-md-end mx-md-1" loading="lazy" />'
         )
 
-    def img_card(name, firstprint=True, front=True):
-        card = CARDS[_name(name)]
-        if "faces" in card:
-            if not front:
-                card = get_card(name, firstprint)["data"][0]
-                return card["card_faces"][1]["image_uris"]["png"]
-            card = get_card(name, firstprint)["data"][0]
-            return card["card_faces"][0]["image_uris"]["png"]
-        if firstprint:
-            card = get_card(name, firstprint)["data"][0]
-            return card["image_uris"]["png"]
-        return f"https://api.scryfall.com/cards/{card['id']}?format=image"
+    def link_card(name):
+        fuzzy = re.sub(r"[^\w\s]", "", name).lower()
+        fuzzy = re.sub(r"\s+", "-", fuzzy)
+        url = f"https://api.scryfall.com/cards/named?exact={fuzzy}"
+        r = requests.get(url)
+        return r.json()["scryfall_uri"]
 
     def card_link(name):
         return flask.Markup(
-            '<a target="_blank" class="text-reset text-decoration-underline" '
+            '<a target="_blank" class="card-name text-decoration-underline" '
             + 'rel="noreferrer" href="'
-            + get_card(name)["data"][0]["scryfall_uri"]
+            + link_card(name)
             + '">'
-            + name
+            + re.sub(r"\s+", "\xA0", name)
+            + "</a>"
+        )
+
+    def card_hover(name):
+        # Il faudra ajouter la possibilité de gérer le hover + créer le DOM
+        return flask.Markup(
+            '<a target="_blank" class="card-name text-decoration-none" '
+            + 'rel="noreferrer" href="'
+            + link_card(name)
+            + '">'
+            + re.sub(r"\s+", "\xA0", name)
             + "</a>"
         )
 
@@ -269,7 +301,9 @@ def display_card():
         deck_name=card_name_from_page,
         img_crop=img_crop,
         img_card=img_card,
+        card_image=card_image,
         card_link=card_link,
+        card_hover=card_hover,
     )
 
 
@@ -277,11 +311,41 @@ def display_card():
 def display_match():
     def match_name(page):
         try:
-            if navigation.HELPER.get(page, {}).get("self").path != "":
-                name = navigation.HELPER.get(page, {}).get("self").name
+            if HELPER.get(page, {}).get("self").path != "":
+                name = HELPER.get(page, {}).get("self").name
                 return name.split(" ", 1)[1]
         except AttributeError:
             pass
         return "Barrin's Codex"
 
     return dict(match_name=match_name)
+
+
+@app.context_processor
+def players():
+    def player_name(page):
+        try:
+            if HELPER.get(page, {}).get("self").path != "":
+                name = HELPER.get(page, {}).get("self").name
+                if name == "Apparitions":
+                    return "Guest"
+                return name.split(" ", 1)[0]
+        except AttributeError:
+            pass
+        return "Barrin's Codex"
+
+    def player_nickname(page):
+        try:
+            if HELPER.get(page, {}).get("self").path != "":
+                name = HELPER.get(page, {}).get("self").name
+                name = name.split(" ", 1)
+                if len(name) == 1:
+                    if name[0] == "Apparitions":
+                        return "Pilote mystérieux·euse"
+                    return name[0]
+                return name[1][2:-2]
+        except AttributeError:
+            pass
+        return "Barrin's Codex"
+
+    return dict(player_name=player_name, player_nickname=player_nickname)
